// MessageHelper.cpp : Implementation of CMessageHelper

#include "stdafx.h"

#define USES_IID_IMessage
#define INITGUID
#include <initguid.h>
#include <mapiutil.h>

#include "assert.h"
#include "cdo.h"						// for our CDO wrapper classes
#include "CdoHelper.h"
#include "MessageHelper.h"
#include "Util.h"
#include "Resource.h"

/////////////////////////////////////////////////////////////////////////////
// CMessageHelper

STDMETHODIMP CMessageHelper::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IMessageHelper
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CMessageHelper::HasTextRTF(
	IDispatch *pdispCdoMessage,
	VARIANT_BOOL *pbRetVal
)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	try
	{
		CCdoMessage CdoMessage;
		CCdoFields CdoFields;
		VARIANT varFields;
		VARIANT varField;
		VARIANT varIndex;
		VARIANT varPropsetID;

		// Associate our CCdoMessage wrapper object with the COM object
		// passed in.
		CdoMessage.AttachDispatch(pdispCdoMessage, FALSE);

		// Obtain the message object's Fields collection.
		varFields = CdoMessage.GetFields();
		assert(varFields.vt == VT_DISPATCH);
		CdoFields.AttachDispatch(varFields.pdispVal);

		// Set up the variants required for obtaining a Field object.
		VariantInit(&varIndex);
		VariantInit(&varPropsetID);
		varIndex.vt = VT_UI4;
		varIndex.ulVal = PR_RTF_COMPRESSED;

		// Obtain the Field object representing the RTF property.
		// An exception is thrown if there is no RTF.
		try
		{
			varField = CdoFields.GetItem(varIndex, varPropsetID);
			assert(varField.vt == VT_DISPATCH);
			*pbRetVal = VARIANT_TRUE;
		}
		catch(...)
		{
			*pbRetVal = VARIANT_FALSE;
		}

		return S_OK;

	}
	catch(...)
	{
		// Unknown error.

		*pbRetVal = VARIANT_FALSE;
		SetAutomationError(IID_IMessageHelper, IDS_MESSAGEHELPER_PROGID,
			IDS_E_UNEXPECTED, IDS_EMPTY, 0);
		return E_FAIL;

	}

} // CMessageHelper::HasTextRTF

STDMETHODIMP CMessageHelper::GetTextRTF(
	IDispatch *pdispCdoMessage,
	BSTR *pbstrRetVal
)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VARIANT varUnkMessage;
	IUnknown *pUnkMessage;
	LPMESSAGE pMessage;
	IStream *pStreamCompressedRTF;
	IStream *pStreamRTF;
	LPSPropValue pvalStoreSupportMask = NULL;

	VariantInit(&varUnkMessage);

	try
	{
		HRX hrx;
		CCdoMessage CdoMessage;

		// Associate our CCdoMessage wrapper object with the COM object
		// passed in.
		CdoMessage.AttachDispatch(pdispCdoMessage, FALSE);

		// Get an IMessage interface so that we can deal with MAPI directly.
		//
		// The accessor method returns a variant.
		varUnkMessage = CdoMessage.GetMAPIOBJECT();
		assert(VT_UNKNOWN==varUnkMessage.vt);
		if (VT_UNKNOWN!=varUnkMessage.vt)
			throw(E_UNEXPECTED);
		//
		// Get the IUnknown pointer from the variant.
		pUnkMessage = varUnkMessage.punkVal;
		assert(NULL!=pUnkMessage);
		if (NULL==pUnkMessage)
			throw(E_UNEXPECTED);
		pUnkMessage->AddRef();
		//
		// Now query for an IMessage interface.
		hrx = pUnkMessage->QueryInterface(IID_IMessage, (void **)&pMessage);

		// Open property containing compressed RTF 
		hrx = pMessage->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, 0, 
			MAPI_DEFERRED_ERRORS, (LPUNKNOWN *)&pStreamCompressedRTF);

		// Some store providers don't support compressed RTF--they just
		// store the uncompressed RTF. Find out if this is one of those.
		hrx = HrGetOneProp(pMessage, PR_STORE_SUPPORT_MASK,
			&pvalStoreSupportMask);
		assert(PR_STORE_SUPPORT_MASK==pvalStoreSupportMask->ulPropTag);

		// Get a stream that has the uncompressed RTF.
		hrx = WrapCompressedRTFStream(pStreamCompressedRTF,
			STORE_UNCOMPRESSED_RTF & pvalStoreSupportMask->Value.l,
			&pStreamRTF);

		// Copy from the uncompressed stream to a BSTR that we can return.
		*pbstrRetVal = Stream2BSTR(pStreamRTF);

		// Release our resources.
		if (pvalStoreSupportMask)
			MAPIFreeBuffer(pvalStoreSupportMask);
		SafeRelease((IUnknown **)&pStreamRTF);
		SafeRelease((IUnknown **)&pStreamCompressedRTF);
		SafeRelease((IUnknown **)&pUnkMessage);
		SafeRelease((IUnknown **)&pMessage);
		VariantClear(&varUnkMessage);

		return S_OK;

	}
	catch(...)
	{
		// Unknown error.

		// Release our resources.
		if (pvalStoreSupportMask)
			MAPIFreeBuffer(pvalStoreSupportMask);
		SafeRelease((IUnknown **)&pStreamRTF);
		SafeRelease((IUnknown **)&pStreamCompressedRTF);
		SafeRelease((IUnknown **)&pUnkMessage);
		SafeRelease((IUnknown **)&pMessage);
		VariantClear(&varUnkMessage);

		*pbstrRetVal = NULL;
		SetAutomationError(IID_IMessageHelper, IDS_MESSAGEHELPER_PROGID,
			IDS_E_UNEXPECTED, IDS_EMPTY, 0);
		return E_FAIL;

	}

} // CMessageHelper::GetTextRTF

STDMETHODIMP CMessageHelper::SetTextRTF(
	IDispatch *pdispCdoMessage,
	BSTR bstrTextRTF
)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VARIANT varUnkMessage;
	IUnknown *pUnkMessage = NULL;
	LPMESSAGE pMessage = NULL;
	IStream *pStreamCompressedRTF = NULL;
	IStream *pStreamRTF = NULL;
	LPSPropValue pvalStoreSupportMask = NULL;

	try
	{
		HRX hrx;
		CCdoMessage CdoMessage;

		// Associate our CCdoMessage wrapper object with the COM object
		// passed in.
		CdoMessage.AttachDispatch(pdispCdoMessage, FALSE);

		// Get an IMessage interface so that we can deal with MAPI directly.
		//
		// The accessor method returns a variant.
		varUnkMessage = CdoMessage.GetMAPIOBJECT();
		assert(VT_UNKNOWN==varUnkMessage.vt);
		if (VT_UNKNOWN!=varUnkMessage.vt)
			throw(E_UNEXPECTED);
		//
		// Get the IUnknown pointer from the variant.
		pUnkMessage = varUnkMessage.punkVal;
		assert(NULL!=pUnkMessage);
		if (NULL==pUnkMessage)
			throw(E_UNEXPECTED);
		//
		// Now query for an IMessage interface.
		hrx = pUnkMessage->QueryInterface(IID_IMessage, (void **)&pMessage);

		// Open the PR_RTF_COMPRESSED property.
		hrx = pMessage->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, 0,
			MAPI_MODIFY | MAPI_CREATE, (LPUNKNOWN *)&pStreamCompressedRTF);

		// Some store providers don't support compressed RTF--they just
		// store the uncompressed RTF. Find out if this is one of those.
		hrx = HrGetOneProp(pMessage, PR_STORE_SUPPORT_MASK,
			&pvalStoreSupportMask);

		// Get IStream pointer for uncompressed RTF, to which we will write.
		hrx = WrapCompressedRTFStream(pStreamCompressedRTF, MAPI_MODIFY 
			| (STORE_UNCOMPRESSED_RTF & pvalStoreSupportMask->Value.l),
			&pStreamRTF);

		// Write the text that was passed to the uncompressed RTF stream.
		hrx = BSTR2Stream(bstrTextRTF, pStreamRTF);

		// Commit changes to the stream.
		hrx = pStreamRTF->Commit(STGC_OVERWRITE);

		// Release our resources.
		if (pvalStoreSupportMask)
			MAPIFreeBuffer(pvalStoreSupportMask);
		SafeRelease((IUnknown **)&pStreamRTF);
		SafeRelease((IUnknown **)&pStreamCompressedRTF);
		SafeRelease((IUnknown **)&pUnkMessage);
		SafeRelease((IUnknown **)&pMessage);

		return S_OK;

	}
	catch(...)
	{
		// Unknown error.

		// Release our resources.
		if (pvalStoreSupportMask)
			MAPIFreeBuffer(pvalStoreSupportMask);
		SafeRelease((IUnknown **)&pStreamRTF);
		SafeRelease((IUnknown **)&pStreamCompressedRTF);
		SafeRelease((IUnknown **)&pUnkMessage);
		SafeRelease((IUnknown **)&pMessage);

		SetAutomationError(IID_IMessageHelper, IDS_MESSAGEHELPER_PROGID,
			IDS_E_UNEXPECTED, IDS_EMPTY, 0);
		return E_FAIL;

	}

} // CMessageHelper::SetTextRTF
