// PropHelper.cpp : Implementation of CPropHelper
#include "stdafx.h"

#define USES_IID_IMAPIProp
#define INITGUID
#include <initguid.h>
#include <mapiutil.h>

#include "assert.h"
#include "cdo.h"						// for our CDO wrapper classes
#include "CdoHelper.h"
#include "PropHelper.h"
#include "Util.h"
#include "Resource.h"

/////////////////////////////////////////////////////////////////////////////
// CPropHelper

STDMETHODIMP CPropHelper::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IPropHelper
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPropHelper::GetNameFromID(
	IDispatch *pdispCdoObject,
	long PropID,
	BSTR *pbstrRetVal
)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	EXCEPINFO excepinfo;
	VARIANT varUnkMAPIObject;
	IUnknown *pUnkMAPIObject = NULL;
	LPMAPIPROP pMAPIProp = NULL;
	LPSPropTagArray pPropTagArray = NULL;
	LPMAPINAMEID *apMAPINameID = NULL;
	ULONG ulNameCount = 0;

	VariantInit(&varUnkMAPIObject);

	try
	{
		HRX hrx;

		// We need an IMAPIProp interface on the underlying MAPI object
		// in order to call its GetNamesFromIDs method. We can't get that
		// interface by QI'ing the passed-in IDispatch, because that
		// IDispatch is for the CDO object, not the MAPI object. Instead,
		// we must read the MAPIOBJECT property through the IDispatch
		// interface. This gives us an IUnknown interface on the underlying
		// MAPI object. We can then QI for the IMAPIProp interface.

		// Get the dispid for the MAPIOBJECT property.
		DISPID dispid;
		OLECHAR *szName = L"MAPIOBJECT";
		hrx = pdispCdoObject->GetIDsOfNames(
			IID_NULL,				// must be IID_NULL
			&szName,				// name of method
			1,						// number of names
			GetUserDefaultLCID(),	// localization info
			&dispid					// dispatch id
		);

		// We're not passing any arguments to the MAPIOBJECT property get
		// method, but we still have to prepare an argument info structure.
		DISPPARAMS dispparamsNoArgs = {
			NULL,					// array of arguments
			NULL,					// dispids of named arguments
			0,						// number of arguments
			0						// number of named arguments
		};

		// Now we're ready to read the MAPIOBJECT property using its dispid.
		hrx = pdispCdoObject->Invoke(
			dispid,					// dispatch id
			IID_NULL,				// must be IID_NULL
			GetUserDefaultLCID(),	// localization info
			DISPATCH_PROPERTYGET,	// reading a property
			&dispparamsNoArgs,		// arguments
			&varUnkMAPIObject,		// return value
			&excepinfo,				// in case there's an exception
			NULL					// [out] index of argument having error
		);

		// At this point, varUnkMAPIObject should contain a pointer to IUnknown.
		assert(VT_UNKNOWN==varUnkMAPIObject.vt);
		if (VT_UNKNOWN!=varUnkMAPIObject.vt)
			throw(E_UNEXPECTED);

		// Get the IUnknown pointer from the variant.
		pUnkMAPIObject = varUnkMAPIObject.punkVal;
		assert(NULL!=pUnkMAPIObject);
		if (NULL==pUnkMAPIObject)
			throw(E_UNEXPECTED);
		pUnkMAPIObject->AddRef();

		// Now query for an IMAPIProp interface.
		hrx = pUnkMAPIObject->QueryInterface(IID_IMAPIProp, (void **)&pMAPIProp);
		assert(NULL!=pMAPIProp);

		// Ok! Now we have an IMAPIProp interface. Let's use it.

		// The property tag is passed to IMAPIProp.GetNamesFromIDs in an
		// SPropTagArray structure.
		hrx = MAPIAllocateBuffer(CbNewSPropTagArray(1),
			reinterpret_cast<void **>(&pPropTagArray));
		assert(NULL!=pPropTagArray);
		if (NULL==pPropTagArray)
			throw(E_OUTOFMEMORY);
		pPropTagArray->cValues = 1;
		pPropTagArray->aulPropTag[1] = static_cast<ULONG>(PropID);

		// Make the call to GetNamesFromIDs.
		hrx = pMAPIProp->GetNamesFromIDs(
			&pPropTagArray,	// properties for which to get names
			NULL,			// pointer to property set GUID
			0,				// flags
			&ulNameCount,	// [out] number of names returned
			&apMAPINameID	// [out] array of pointers to MAPINAMEID structures
		);

		*pbstrRetVal = SysAllocString(L"Looks like GetNamesFromIDs worked!");

		// Release our resources.
		MAPIFreeBuffer(static_cast<void *>(pPropTagArray));
		pPropTagArray = NULL;
		if (ulNameCount>0)
		{
			MAPIFreeBuffer(static_cast<void *>(apMAPINameID));
			apMAPINameID = NULL;
		}
		SafeRelease((IUnknown **)&pMAPIProp);
		SafeRelease((IUnknown **)&pUnkMAPIObject);
		VariantClear(&varUnkMAPIObject);

		return S_OK;

	}
	catch(HRESULT hr)
	{
		// Release our resources.
		MAPIFreeBuffer(static_cast<void *>(pPropTagArray));
		pPropTagArray = NULL;
		if (ulNameCount>0)
		{
			MAPIFreeBuffer(static_cast<void *>(apMAPINameID));
			apMAPINameID = NULL;
		}
		SafeRelease((IUnknown **)&pMAPIProp);
		SafeRelease((IUnknown **)&pUnkMAPIObject);
		VariantClear(&varUnkMAPIObject);

		// Clear the return value.
		*pbstrRetVal = NULL;

		// Did our call to IDispatch.Invoke fail?
		if (DISP_E_EXCEPTION==hr)
		{
			// Server is permitted to defer filling EXCEPINFO.
			if (NULL!=excepinfo.pfnDeferredFillIn)
			{
				// Fill the EXCEPINFO structure.
				(*(excepinfo.pfnDeferredFillIn))(&excepinfo);
			}
			
			// Set the automation error based on what the
			// EXCEPINFO says.
			SetAutomationError(
				IID_IPropHelper,
				static_cast<LPCOLESTR>(excepinfo.bstrSource),
				static_cast<LPCOLESTR>(excepinfo.bstrDescription),
				static_cast<LPCOLESTR>(excepinfo.bstrHelpFile),
				excepinfo.dwHelpContext
			);
			return excepinfo.scode;
		}
		else if (E_INVALIDARG==hr)
		{
			SetAutomationError(IID_IPropHelper, IDS_PROPHELPER_PROGID,
				IDS_E_NOT_NAMED_PROPERTY, IDS_EMPTY, 0);
			return E_INVALIDARG;
		}
		else if (E_OUTOFMEMORY==hr)
		{
			SetAutomationError(IID_IPropHelper, IDS_PROPHELPER_PROGID,
				IDS_E_OUTOFMEMORY, IDS_EMPTY, 0);
			return E_OUTOFMEMORY;
		}
		// else unknown error
		else
		{
			SetAutomationError(IID_IPropHelper, IDS_PROPHELPER_PROGID,
				IDS_E_UNEXPECTED, IDS_EMPTY, 0);
			return E_FAIL;
		}
	} // catch(HRESULT hr)
	catch(...)
	{
		// Unknown error.

		// Release our resources.
		MAPIFreeBuffer(static_cast<void *>(pPropTagArray));
		pPropTagArray = NULL;
		if (ulNameCount>0)
		{
			MAPIFreeBuffer(static_cast<void *>(apMAPINameID));
			apMAPINameID = NULL;
		}
		SafeRelease((IUnknown **)&pMAPIProp);
		SafeRelease((IUnknown **)&pUnkMAPIObject);
		VariantClear(&varUnkMAPIObject);

		*pbstrRetVal = NULL;
		SetAutomationError(IID_IPropHelper, IDS_PROPHELPER_PROGID,
			IDS_E_UNEXPECTED, IDS_EMPTY, 0);
		return E_FAIL;

	}
}
