Attribute VB_Name = "Globals"
Option Explicit

Public Enum ErrorCodes
   CancelError = vbObjectError + 1
End Enum

' This constant will be used to mean all recipient types.
Public Const gcAllRecipientTypes As Long = -1

' This is the CDO session through which all code talks to MAPI.
Public gCdoSession As MAPI.Session

' This is the number of twips between a control and the edge of the form.
' It is half the number of twips between two controls.
Public Const gcsngMargin As Single = 30

' Declarations needed for making a window to be "always on top."

' SetWindowPos Function
Declare Function SetWindowPos Lib "user32" _
   (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, _
   ByVal x As Long, ByVal y As Long, ByVal cx As Long, _
   ByVal cy As Long, ByVal wFlags As Long) As Long
   
' SetWindowPos Flags
Public Const SWP_NOSIZE = &H1
Public Const SWP_NOMOVE = &H2
Public Const HWND_TOPMOST = -1
Public Const HWND_NOTOPMOST = -2

' For the Debug version of the DLL, remark the following line
 Public Declare Function WriteRTF _
    Lib "mapirtf.dll" _
    Alias "writertf" (ByVal ProfileName As String, ByVal MessageID As String, _
                        ByVal StoreID As String, ByVal cText As String) _
    As Integer
 Public Declare Function ReadRTF _
    Lib "mapirtf.dll" _
    Alias "readrtf" (ByVal ProfileName As String, ByVal SrcMsgID As String, _
                        ByVal SrcStoreID As String, ByRef MsgRTF As String) _
    As Integer
    
' For the Debug version of the DLL, un-remark the following line
'Public Declare Function WriteRTF _
    Lib "mapirtfd.dll" _
    Alias "writertf" (ByVal ProfileName As String, ByVal MessageID As String, _
                        ByVal StoreID As String, ByVal cText As String) _
    As Integer
'Public Declare Function ReadRTF _
    Lib "mapirtfd.dll" _
    Alias "readrtf" (ByVal ProfileName As String, ByVal SrcMsgID As String, _
                        ByVal SrcStoreID As String, ByRef MsgRTF As String) _
    As Integer

Public Sub gCenterForm(frm As Form)

   ' Center the given form within the screen.
   
   frm.Left = (Screen.Width - frm.Width) \ 2
   frm.Top = (Screen.Height - frm.Height) \ 2

End Sub ' gCenterForm

Public Sub gSetAlwaysOnTop(frm As Form)

   ' Set the given form to be always on top.
   Call SetWindowPos(frm.hwnd, HWND_TOPMOST, 0, 0, 0, 0, _
      SWP_NOSIZE Or SWP_NOMOVE)

End Sub ' gSetAlwaysOnTop

Public Sub Main()

   ' Entry point for the application.
   
   ' Show the splash screen.
   frmSplash.Show vbModeless
   
   ' Show the main form.
   frmMain.Show
   
   ' Tell the splash screen it can unload when it's ready.
   frmSplash.ProgramInitializationDone = True

End Sub ' Main

Public Function gNoNeg(vnt As Variant) As Variant

   ' Return 0 or vnt, whichever is higher.  Used by form resize event handlers.
   
   If vnt < 0 Then
      gNoNeg = 0
   Else
      gNoNeg = vnt
   End If

End Function ' gNoNeg

Public Sub gOpenCdoSession()

   ' If there is currently an open session, close it before opening
   ' another one.
   If Not (gCdoSession Is Nothing) Then
      gCloseCdoSession
   End If
   
   ' Instantiate a new session object and logon.
   Set gCdoSession = New MAPI.Session
   gCdoSession.Logon

End Sub ' gOpenCdoSession

Public Sub gCloseCdoSession()

   ' Log off of MAPI.
   If Not (gCdoSession Is Nothing) Then
      gCdoSession.Logoff
      Set gCdoSession = Nothing
   End If

End Sub ' gCloseCdoSession

Public Function gRecipientListString( _
   ByRef CdoRecipients As MAPI.Recipients, _
   Optional ByVal nType As Long = gcAllRecipientTypes) As String
   
   ' Given the collection of recipients, return a semicolon-delimited
   ' list of recipient names.
   
   Dim strReturn As String
   Dim CdoRecipient As MAPI.Recipient
   
   strReturn = ""
   
   For Each CdoRecipient In CdoRecipients
      If nType = gcAllRecipientTypes Or nType = CdoRecipient.Type Then
         strReturn = strReturn & CdoRecipient.Name & "; "
      End If
   Next CdoRecipient
   
   If strReturn <> "" Then
      strReturn = Left(strReturn, Len(strReturn) - 2)
   End If
   
   gRecipientListString = strReturn
   
End Function ' gRecipientListString

Public Function gGetDateReceivedSafe( _
   ByRef CdoMessage As MAPI.Message) As String
   
   Dim dtTimeReceived As Date
   
   On Error GoTo ErrorHandler
   
   dtTimeReceived = CDate(CdoMessage.TimeReceived)
   gGetDateReceivedSafe = Format(dtTimeReceived, _
      "ddd m/d/yy h:mm AMPM")
   
   Exit Function
   
ErrorHandler:
   
   gGetDateReceivedSafe = ""
   
End Function ' gGetDateReceivedSafe

Public Function gGetDateSentSafe(ByRef CdoMessage As MAPI.Message) _
   As String
   
   Dim dtTimeSent As Date
   
   On Error GoTo ErrorHandler
   
   dtTimeSent = CDate(CdoMessage.TimeSent)
   gGetDateSentSafe = Format(dtTimeSent, "ddd m/d/yy h:mm AMPM")
   
   Exit Function
   
ErrorHandler:
   
   gGetDateSentSafe = ""
   
End Function ' gGetDateSentSafe

Public Function gGetRtfTextSafe(ByRef CdoMessage As MAPI.Message) _
   As String

   On Error GoTo ErrorHandler
   
   Dim MessageHelper As CdoHelper.MessageHelper
   Set MessageHelper = New CdoHelper.MessageHelper
   gGetRtfTextSafe = MessageHelper.GetTextRTF(CdoMessage)
   Set MessageHelper = Nothing

   Exit Function

ErrorHandler:

   gGetRtfTextSafe = ""

End Function ' gGetRtfTextSafe

Public Sub gReloadMessage(ByRef CdoMessage As MAPI.Message)

   ' Reload the given message from the message store.
   
   Dim CdoSession As MAPI.Session
   Dim CdoFolder As MAPI.Folder
   Dim CdoMessages As MAPI.Messages
   Dim CdoMessageFilter As MAPI.MessageFilter
   
   ' Get the Filter object for the Messages collection in which
   ' the given message resides.
   Set CdoSession = CdoMessage.Session
   Set CdoFolder = CdoSession.GetFolder(CdoMessage.FolderID)
   Set CdoMessages = CdoFolder.Messages
   Set CdoMessageFilter = CdoMessages.Filter
   
   ' Set the Filter object to filter for our specific message.
   CdoMessageFilter.Fields(CdoPR_ENTRYID) = CdoMessage.ID
   
   ' Get the message.
   Set CdoMessage = CdoMessages.GetFirst
   
   ' Turn the filter off.
   Set CdoMessages.Filter = Nothing
   
   ' Release our references.
   Set CdoMessageFilter = Nothing
   Set CdoMessages = Nothing
   Set CdoFolder = Nothing
   Set CdoSession = Nothing

End Sub ' gReloadMessage

Public Sub gSetMessageTextRtf(ByRef CdoMessage As MAPI.Message, _
   strTextRTF As String)
   
   Dim MessageHelper As CdoHelper.MessageHelper
   Set MessageHelper = New CdoHelper.MessageHelper
   
   MessageHelper.SetTextRTF CdoMessage, strTextRTF
   
   Set MessageHelper = Nothing
   
   Exit Sub
   
'   Dim CdoFolder As MAPI.Folder
'   Dim CdoMessages As MAPI.Messages
'   Dim CdoMessageFilter As MAPI.MessageFilter
'   Dim i As Integer
'
'   ' The message must be in the message store in order for the
'   ' WriteRTF helper function to find it.
'   CdoMessage.Update MakePermanent:=True, RefreshObject:=True
'
''   i = WriteRTF(CdoMessage.Session.Name, CdoMessage.ID, _
''      CdoMessage.StoreID, strTextRTF)
''   Debug.Assert i = 0
'
'   CdoMessage.Fields(CdoPR_RTF_COMPRESSED) = gstrDebug
'   CdoMessage.Update MakePermanent:=True, RefreshObject:=True
'
''   ' The WriteRTF helper function modifies the message in the
''   ' message store, but our message object won't reflect the
''   ' change until we reload it from the message store.
''   gReloadMessage CdoMessage

End Sub ' gSetMessageTextRtf

Public Sub gSetRecipients(ByRef CdoMessage As MAPI.Message, _
   ByVal strRecipients As String, _
   CdoType As MAPI.CdoRecipientType)
   
   ' Create recipients on the message, based on the semicolon-delimited
   ' string of recipient names.
   
   Dim CdoRecipients As MAPI.Recipients
   Dim CdoRecipient As MAPI.Recipient
   
   On Error GoTo ErrorHandler
   
   Set CdoRecipients = CdoMessage.Recipients
   
   ' Clear the recipient list.
   CdoRecipients.Delete
   
   Dim astrNames() As String
   Dim n As Long
   
   ' Get the names.
   gParse strRecipients, ";", astrNames()
   
   ' Create a recipient for each name.
   For n = LBound(astrNames) To UBound(astrNames)
      CdoRecipients.Add Name:=astrNames(n), Type:=CdoType
   Next n
   
   ' Resolve the names.
   CdoRecipients.Resolve
   
   ' Done
   Set CdoRecipients = Nothing
   
   Exit Sub
   
ErrorHandler:

   If Err.Number = CdoE_USER_CANCEL Then
      Resume Next
   Else
      Err.Raise Err.Number
   End If
   
End Sub ' gSetRecipients

Public Function gGetRecipientNames(ByRef CdoRecipients _
   As MAPI.Recipients, nRecipientType As Long) As String
   
   ' Return a string with all of the display names of the recipients,
   ' separated by semicolons.
   
   Dim CdoRecipient As MAPI.Recipient
   Dim strNames As String
   
   strNames = ""
   For Each CdoRecipient In CdoRecipients
      If (nRecipientType = gcAllRecipientTypes) _
         Or (nRecipientType = CdoRecipient.Type) Then
         strNames = strNames & CdoRecipient.Name & "; "
      End If
   Next CdoRecipient
   
   ' Remove the extra semicolon.
   If Len(strNames) > 0 Then
      strNames = Left(strNames, Len(strNames) - 2)
   End If
   
   gGetRecipientNames = strNames
   
End Function ' gGetRecipientNames

Public Sub gParse(ByVal strSource As String, _
   ByVal strDelimiter As String, ByRef astrElements() As String)
   
   Dim nCount As Long
   Dim nPosStart As Long
   Dim nPosDelimiter As Long
   
   ' This makes the code easier.
   strSource = strSource & strDelimiter
   
   nCount = 0
   nPosStart = 1
   
   nPosDelimiter = InStr(nPosStart, strSource, strDelimiter)
   Do While nPosDelimiter > 0
      nCount = nCount + 1
      ReDim Preserve astrElements(1 To nCount) As String
      astrElements(nCount) = _
         Trim(Mid(strSource, nPosStart, nPosDelimiter - nPosStart))
      nPosStart = nPosDelimiter + 1
      nPosDelimiter = InStr(nPosStart, strSource, strDelimiter)
   Loop

End Sub ' gParse

Public Function gGetSenderSafe(ByRef CdoMessage As MAPI.Message) _
   As String

   On Error GoTo ErrorHandler
   
   gGetSenderSafe = CdoMessage.Sender
   
   Exit Function
   
ErrorHandler:
   
   gGetSenderSafe = ""

End Function ' gGetSenderSafe
